@@ -3,7 +3,7 @@ use core::fmt::Write;
 use core::ops::Deref;
 use core::{fmt, mem, ptr, str};
 
-use crate::inline::MAX_INLINE;
+use crate::inline::STRING_SIZED_INLINE;
 
 // The size of internal buffer for formatting (if larger needed we punt and just use a heap allocated String)
 pub(crate) const BUFFER_SIZE: usize = 1024;
@@ -31,7 +31,7 @@ impl<const N: usize> StringBuffer<N> {
     }
 
     #[inline]
-    pub fn capacity(&self) -> usize {
+    pub fn capacity() -> usize {
         N
     }
 
@@ -91,7 +91,7 @@ impl<const N: usize> StringBuffer<N> {
     pub fn write(&mut self, s: &str) -> bool {
         let len = self.len();
 
-        if (self.capacity() - len) >= s.len() {
+        if (Self::capacity() - len) >= s.len() {
             let buffer = &mut self.buffer[len..];
 
             unsafe {
@@ -125,13 +125,14 @@ impl<const N: usize> Deref for StringBuffer<N> {
 // *** FlexStr Builder ***
 
 #[allow(clippy::large_enum_variant)]
-pub(crate) enum FlexStrBuilder {
-    Small(StringBuffer<MAX_INLINE>),
-    Regular(StringBuffer<BUFFER_SIZE>),
+pub(crate) enum FlexStrBuilder<const N: usize = STRING_SIZED_INLINE, const N2: usize = BUFFER_SIZE>
+{
+    Small(StringBuffer<N>),
+    Regular(StringBuffer<N2>),
     Large(String),
 }
 
-impl FlexStrBuilder {
+impl<const N: usize, const N2: usize> FlexStrBuilder<N, N2> {
     #[inline]
     pub fn new() -> Self {
         // TODO: Is it worth assuming inline size if we don't know the capacity needed???
@@ -140,9 +141,9 @@ impl FlexStrBuilder {
 
     #[inline]
     pub fn with_capacity(cap: usize) -> Self {
-        if cap <= MAX_INLINE {
+        if cap <= N {
             FlexStrBuilder::Small(StringBuffer::new())
-        } else if cap <= BUFFER_SIZE {
+        } else if cap <= N2 {
             FlexStrBuilder::Regular(StringBuffer::new())
         } else {
             FlexStrBuilder::Large(String::with_capacity(cap))
@@ -150,10 +151,7 @@ impl FlexStrBuilder {
     }
 
     #[inline]
-    fn create_string_and_write<const N: usize>(
-        buffer: &mut StringBuffer<N>,
-        s: &str,
-    ) -> FlexStrBuilder {
+    fn create_string_and_write<const N3: usize>(buffer: &mut StringBuffer<N3>, s: &str) -> Self {
         let required_cap = buffer.len() + s.len();
         // Start with a capacity twice the size of what is needed (to try and avoid future heap allocations)
         let mut buffer = buffer.to_string_buffer(required_cap * 2);
@@ -180,7 +178,7 @@ impl FlexStrBuilder {
     }
 }
 
-impl Write for FlexStrBuilder {
+impl<const N: usize, const N2: usize> Write for FlexStrBuilder<N, N2> {
     fn write_str(&mut self, s: &str) -> fmt::Result {
         match self {
             FlexStrBuilder::Small(buffer) => {
@@ -210,20 +208,40 @@ impl Write for FlexStrBuilder {
     }
 }
 
+// NOTE: This is a macro because one of the variants is HUGE and we can't take a chance that the
+// optimizer won't inline the call as that will copy a LOT of data unnecessarily (since we are
+// copying it again regardless for `Regular` and `Large` variants)
+macro_rules! builder_into {
+    ($builder:ident) => {
+        match $builder {
+            $crate::builder::FlexStrBuilder::Small(buffer) => {
+                let len: u8 = buffer.len() as u8;
+                FlexStr($crate::FlexStrInner::Inlined(
+                    $crate::inline::InlineFlexStr::from_array(buffer.into_inner(), len),
+                ))
+            }
+            $crate::builder::FlexStrBuilder::Regular(buffer) => {
+                $crate::traits::ToFlex::to_flex(&*buffer)
+            }
+            $crate::builder::FlexStrBuilder::Large(s) => s.into(),
+        }
+    };
+}
+
 #[cfg(test)]
 mod tests {
     use alloc::string::ToString;
     use core::fmt::Write;
 
     use crate::builder::{FlexStrBuilder, StringBuffer, BUFFER_SIZE};
-    use crate::inline::MAX_INLINE;
+    use crate::inline::STRING_SIZED_INLINE;
     use crate::FlexStr;
 
     #[test]
     fn string_buffer() {
         // Write 1
         let write1 = "test";
-        let mut buffer: StringBuffer<MAX_INLINE> = StringBuffer::new();
+        let mut buffer: StringBuffer<STRING_SIZED_INLINE> = StringBuffer::new();
         assert!(buffer.is_empty());
         assert!(buffer.write(write1));
         assert_eq!(buffer.len(), write1.len());
@@ -266,7 +284,7 @@ mod tests {
     fn flex_str_builder_promotion() {
         // Write 1 - verify inline buffer size
         let write1 = "test";
-        let mut builder = FlexStrBuilder::new();
+        let mut builder = <FlexStrBuilder>::new();
         assert!(matches!(builder, FlexStrBuilder::Small(_)));
         assert!(builder.write_str(write1).is_ok());
         assert!(matches!(builder, FlexStrBuilder::Small(_)));
@@ -280,7 +298,7 @@ mod tests {
         let write3 = "x".repeat(BUFFER_SIZE);
         assert!(builder.write_str(&write3).is_ok());
         assert!(matches!(builder, FlexStrBuilder::Large(_)));
-        let s: FlexStr = builder.into();
+        let s: FlexStr = builder_into!(builder);
         assert_eq!(s, write1.to_string() + write2 + &write3);
     }
 }
